/*
 * Copyright 2011 Francois ROLAND
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.optimaize.langdetect.frma;

import static org.mockito.Mockito.*;

import java.io.Closeable;
import java.io.IOException;

import org.junit.Test;

public class IOUtilsTest {

	@Test
	public void closeQuietlyNullStream() {
		IOUtils.closeQuietly(null);
	}

	@Test
	public void closeQuietlyWhenExceptionThrown() throws IOException {
		Closeable stream = mock(Closeable.class);
		doThrow(new IOException()).when(stream).close();
		IOUtils.closeQuietly(stream);
	}

	@Test
	public void closeQuietly() throws IOException {
		Closeable stream = mock(Closeable.class);
		IOUtils.closeQuietly(stream);
		verify(stream, times(1)).close();
	}

}
