/**
 * Provides functionality for concatenating and cleaning text that is used as
 * a) learning text to produce {@link com.optimaize.langdetect.LanguageProfile}s
  * b) for the text for which the language is to be guessed.
 *
 * @author Fabian Kessler
 */
package com.optimaize.langdetect.text;